/******************************************************************************
//                                                                           //
//  Project: Software Interface for USB-ADIO Devices                         //
//                                                                           //
//  CGC Instruments, (c) 2009-2017, Version 1.10. All rights reserved.       //
//                                                                           //
//  Definition for USB_ADIO DLL Routines                                     //
//                                                                           //
******************************************************************************/

#ifndef __USB_ADIO_H__
#define __USB_ADIO_H__


#ifdef __cplusplus
extern "C" {
#endif


#define USB_ADIO_MAX_PORT 8 /* Maximum PortNumber */

/*
	The listed routines accept the number of the port PortNumber in the range 0..USB_ADIO_MAX_PORT-1,
	thus maximum USB_ADIO_MAX_PORT independent communication channels (ports) can be defined

	Each communication channel must be opened before the first usage. The Open routine assigns a specified COMx port to the channel.
	The ComNumber specifies the number of the port - 1 stands for COM1, 2 for COM2, etc.

	If necessary, the channel may be closed and assigned by the Open routine to a new COMx port.
	The channels are closed automatically at the end of the program, the main program does not need to take care about it.

	The return values of the routines are the following defines USB_ADIO_ERR_XXX
		 0: routine finished without any error
		 1: USB_ADIO_CheckInput returns 1 if no new data is available
		<0: errors USB_ADIO_ERR_XXX

	The last error code can be obtained by calling USB_ADIO_State.
	The routine USB_ADIO_ErrorMessage returns a pointer to a zero-terminated character string.

	If a communication error occurred, the error code can be red again by USB_ADIO_IO_State,
	The routine USB_ADIO_IO_ErrorMessage provides the corresponding error message.
*/

#define USB_ADIO_ERR_PORT_RANGE       -1 /* PortNumber out of range */
#define USB_ADIO_ERR_OPEN             -3 /* Error opening the port */
#define USB_ADIO_ERR_CLOSE            -4 /* Error closing the port  */
#define USB_ADIO_ERR_PURGE            -5 /* Error purging the port  */
#define USB_ADIO_ERR_CONTROL          -6 /* Error setting the port control lines */
#define USB_ADIO_ERR_STATUS           -7 /* Error reading the port status lines */
#define USB_ADIO_ERR_COMMAND_SEND     -8 /* Error sending command */
#define USB_ADIO_ERR_DATA_SEND        -9 /* Error sending data */
#define USB_ADIO_ERR_TERM_SEND       -10 /* Error sending termination character */
#define USB_ADIO_ERR_COMMAND_RECEIVE -11 /* Error receiving command */
#define USB_ADIO_ERR_DATA_RECEIVE    -12 /* Error receiving data */
#define USB_ADIO_ERR_TERM_RECEIVE    -13 /* Error receiving termination character */
#define USB_ADIO_ERR_COMMAND_WRONG   -14 /* Wrong command received */
#define USB_ADIO_ERR_ARGUMENT_WRONG  -15 /* Wrong argument received */
#define USB_ADIO_ERR_ARGUMENT        -16 /* Wrong argument passed to the function */

#define USB_ADIO_ERR_NOT_CONNECTED -100 /* Device not connected */
#define USB_ADIO_ERR_NOT_READY     -101 /* Device not ready */
#define USB_ADIO_ERR_READY         -102 /* Device state could not be set to not ready */

#define USB_ADIO_ERR_DEBUG_OPEN    -400 /* Error opening the file for debugging output */
#define USB_ADIO_ERR_DEBUG_CLOSE   -401 /* Error closing the file for debugging output */


/* General */

WORD FAR _export __pascal USB_ADIO_Version(); /* Get the USB-ADIO software version */

int  FAR _export __pascal USB_ADIO_Open  (WORD PortNumber, WORD ComNumber); /* Open the port  */
int  FAR _export __pascal USB_ADIO_Close (WORD PortNumber);                 /* Close the port */

int  FAR _export __pascal USB_ADIO_Purge        (WORD PortNumber);               /* Clear data buffers for the port */
int  FAR _export __pascal USB_ADIO_Buffer_State (WORD PortNumber, BOOL & empty); /* Return true if the input data buffer of the device is empty */
int  FAR _export __pascal USB_ADIO_Device_Purge (WORD PortNumber, BOOL & empty); /* Clear output data buffer of the device, return value as USB_ADIO_Buffer_State */
BOOL FAR _export __pascal USB_ADIO_Ret_Buffer_State (WORD PortNumber);           /* Pascal optimized: return true if the input data buffer of the device is empty */
BOOL FAR _export __pascal USB_ADIO_Ret_Device_Purge (WORD PortNumber);           /* Pascal optimized: clear output data buffer of the device, return value as USB_ADIO_Ret_Buffer_State */


/* Analog I/O */

int   FAR _export __pascal USB_ADIO_GetAnalogInput     (WORD PortNumber, BOOL SecondaryIO, float & Voltage); /* Read the AIN0/AIO1 analog input voltage */
int   FAR _export __pascal USB_ADIO_AutoAnalogInput    (WORD PortNumber, BOOL SecondaryIO);                  /* Set automatic transmission of the AIN0/AIO1 analog input voltage */
int   FAR _export __pascal USB_ADIO_GetLastAnalogInput (WORD PortNumber, BOOL SecondaryIO, float & Voltage); /* Get last automatically transmitted AIN0/AIO1 analog input voltage */
float FAR _export __pascal USB_ADIO_RetAnalogInput     (WORD PortNumber, BOOL SecondaryIO);                  /* Pascal optimized: read the AIN0/AIO1 analog input voltage */
float FAR _export __pascal USB_ADIO_RetLastAnalogInput (WORD PortNumber, BOOL SecondaryIO);                  /* Pascal optimized: get last automatically transmitted AIN0/AIO1 analog input voltage */

int   FAR _export __pascal USB_ADIO_SetAnalogOutput    (WORD PortNumber, float   Voltage); /* Set analog output voltage */
int   FAR _export __pascal USB_ADIO_GetAnalogOutput    (WORD PortNumber, float & Voltage); /* Read analog output voltage */
float FAR _export __pascal USB_ADIO_RetAnalogOutput    (WORD PortNumber);                  /* Pascal optimized: read analog output voltage */


/* Digital I/O */

int  FAR _export __pascal USB_ADIO_GetDigitalInput     (WORD PortNumber, WORD & Data); /* Read digital input state */
int  FAR _export __pascal USB_ADIO_AutoDigitalInput    (WORD PortNumber);              /* Set automatic transmission of digital input state */
int  FAR _export __pascal USB_ADIO_GetLastDigitalInput (WORD PortNumber, WORD & Data); /* Get last automatically transmitted digital input state */
WORD FAR _export __pascal USB_ADIO_RetDigitalInput     (WORD PortNumber);              /* Pascal optimized: read digital input state */
WORD FAR _export __pascal USB_ADIO_RetLastDigitalInput (WORD PortNumber);              /* Pascal optimized: get last automatically transmitted digital input state */

int  FAR _export __pascal USB_ADIO_SetDigitalOutput    (WORD PortNumber, WORD   Data); /* Set digital output state */
int  FAR _export __pascal USB_ADIO_GetDigitalOutput    (WORD PortNumber, WORD & Data); /* Read digital output state */
WORD FAR _export __pascal USB_ADIO_RetDigitalOutput    (WORD PortNumber);              /* Pascal optimized: read digital output state */


/* Automatic data input */

#define USB_ADIO_AUTO_AIN  1 /* Automatic analog input data at AIN0 received */
#define USB_ADIO_AUTO_AIO  4 /* Automatic analog input data at AIO1 received */
#define USB_ADIO_AUTO_DIN  2 /* Automatic digital input data received */
int  FAR _export __pascal USB_ADIO_CheckAutoInput (WORD PortNumber, WORD & Mask); /* Check for new automatic notification data */
int  FAR _export __pascal USB_ADIO_GetAutoMask    (WORD PortNumber, WORD & Mask); /* Get the mask of the last automatic notification data */
WORD FAR _export __pascal USB_ADIO_RetAutoInput   (WORD PortNumber);              /* Pascal optimized: check for new automatic notification data */
WORD FAR _export __pascal USB_ADIO_RetAutoMask    (WORD PortNumber);              /* Pascal optimized: get the mask of the last automatic notification data */


/* System */

int   FAR _export __pascal USB_ADIO_GetCPUVoltage     (WORD PortNumber, float & Voltage    ); /* Read CPU supply voltage */
int   FAR _export __pascal USB_ADIO_GetCPUTemperature (WORD PortNumber, float & Temperature); /* Read CPU temperature    */
float FAR _export __pascal USB_ADIO_RetCPUVoltage     (WORD PortNumber);                      /* Pascal optimized: read CPU supply voltage */
float FAR _export __pascal USB_ADIO_RetCPUTemperature (WORD PortNumber);                      /* Pascal optimized: read CPU temperature    */

int FAR _export __pascal USB_ADIO_GetUptime (WORD PortNumber, DWORD & Seconds, WORD & Milliseconds); /* Get device uptime */

int   FAR _export __pascal USB_ADIO_FW_Ver      (WORD PortNumber, WORD & Version);             /* Get the USB-ADIO firmware version */
int   FAR _export __pascal USB_ADIO_FW_Date     (WORD PortNumber, char * FAR DateString);      /* Get the USB-ADIO firmware date */
int   FAR _export __pascal USB_ADIO_prod_ID     (WORD PortNumber, char * FAR Identification);  /* Get the USB-ADIO product identification */
int   FAR _export __pascal USB_ADIO_prod_No     (WORD PortNumber, DWORD & Number);             /* Get the USB-ADIO product number */
WORD  FAR _export __pascal USB_ADIO_Ret_FW_Ver  (WORD PortNumber);                             /* Pascal optimized: get the USB-ADIO firmware version */
DWORD FAR _export __pascal USB_ADIO_Ret_prod_No (WORD PortNumber);                             /* Pascal optimized: get the USB-ADIO product number */


/* Communication port */

int          FAR _export __pascal USB_ADIO_State           (WORD PortNumber); /* Get software interface state */
const char * FAR _export __pascal USB_ADIO_ErrorMessage    (WORD PortNumber); /* Get the error message corresponding to the software interface state */
int          FAR _export __pascal USB_ADIO_IO_State        (WORD PortNumber); /* Get serial port interface state */
const char * FAR _export __pascal USB_ADIO_IO_ErrorMessage (WORD PortNumber); /* Get the error message corresponding to the serial port interface state */


/* Debugging output */

int FAR _export __pascal USB_ADIO_OpenDebugFile  (WORD PortNumber, const char * FAR FileName); /* Open a text file for debugging output, return 0 on success */
int FAR _export __pascal USB_ADIO_CloseDebugFile (WORD PortNumber);                            /* Close the file for debugging output, return 0 on success */

#ifdef __cplusplus
}
#endif


#endif//__USB_ADIO_H__
